#!/usr/bin/env python

import os
import sys

from charmhelpers.contrib.charmsupport import nrpe
from charmhelpers.core.hookenv import (
    Hooks,
    charm_dir,
    close_port,
    config as config_get,
    log as juju_log,
    open_port,
    status_set,
)
from charmhelpers.core.host import (
    CompareHostReleases,
    lsb_release,
    service_restart,
    service_start,
    service_stop,
)
from charmhelpers.fetch import (
    apt_install
)


DEFAULT_RSYSLOG_PORT = 514
DEFAULT_RELP_PORT = 2514
DEFAULT_RSYSLOG_PATH = os.path.join(os.path.sep, 'etc', 'rsyslog.d')
DEFAULT_LOGROTATE_PATH = os.path.join(os.path.sep, 'etc', 'logrotate.d',
                                      'rsyslog')
DEFAULT_CONFIGURATION_PARAMS = (
    'forward_host',
    'forward_options',
    'forward_port',
    'forward_protocol',
    'forward_selector',
    'syslog_rotate',
    'messages_rotate',
    'nova_logs',
    'protocol',
)

required_debian_pkgs = [
    'rsyslog',
    'rsyslog-relp',
    'python-jinja2',
]

hooks = Hooks()

try:
    from jinja2 import Environment, FileSystemLoader
except ImportError:
    apt_install(['python-jinja2'], options=['-f'])
    from jinja2 import Environment, FileSystemLoader


def get_template_dir():
    return os.path.join(charm_dir(), 'templates')


def get_config_template(name):
    template_env = Environment(loader=FileSystemLoader(get_template_dir()))
    return template_env.get_template(name)


def get_changed_config():
    changed = {}
    for config_key in DEFAULT_CONFIGURATION_PARAMS:
        changed[config_key] = config_get(config_key)
        juju_log("Configuration key:%s set to value: %s" %
                 (config_key, changed[config_key]))

    # allows templates to generate series-dependent configuration
    lsb = lsb_release()
    changed['series'] = CompareHostReleases(lsb['DISTRIB_CODENAME'])

    return changed


@hooks.hook()
def install():
    apt_install(required_debian_pkgs, options=['-f'])


@hooks.hook("upgrade-charm")
def upgrade_charm():
    install()


@hooks.hook("start")
def start():
    service_start("rsyslog")

    proto = config_get('protocol')
    if proto == "relp":
        open_port(DEFAULT_RELP_PORT)
    elif proto == "tcp":
        open_port(DEFAULT_RSYSLOG_PORT)
    else:
        open_port(DEFAULT_RSYSLOG_PORT, protocol="UDP")


@hooks.hook("stop")
def stop():
    service_stop("rsyslog")

    proto = config_get('protocol')
    if proto == "relp":
        close_port(DEFAULT_RELP_PORT)
    elif proto == "tcp":
        close_port(DEFAULT_RSYSLOG_PORT)
    else:
        close_port(DEFAULT_RSYSLOG_PORT, protocol="UDP")


def update_logrotate_config(**params):
    template_name = "rsyslog.conf"
    template = get_config_template(template_name)

    with open(DEFAULT_LOGROTATE_PATH, 'w') as logrotate_config:
        logrotate_config.write(template.render(**params))


@hooks.hook("nrpe-external-master-relation-changed")
@hooks.hook("local-monitors-relation-changed")
def update_nrpe_config():
    nrpe_compat = nrpe.NRPE()
    nrpe_compat.add_check(
        shortname="rsyslog",
        description="Check rsyslog is running",
        check_cmd="check_procs -c 1: -C rsyslogd"
    )
    nrpe_compat.write()


def update_rsyslog_config(template_name, **params):
    template = get_config_template(template_name)

    with open(os.path.join(DEFAULT_RSYSLOG_PATH,
                           template_name), 'w') as rsyslog_config:
        rsyslog_config.write(template.render(**params))


@hooks.hook("config-changed")
def config_changed():
    config = get_changed_config()

    update_logrotate_config(**config)
    update_rsyslog_config(template_name="60-aggregator.conf", **config)
    if config_get('forward_host') != "":
        update_rsyslog_config(template_name="70-forward.conf", **config)

    juju_log("Configuration changed, restart required")

    # configuration changed, restart rsyslog
    service_restart("rsyslog")

    update_nrpe_config()
    status_set("active", "Unit is ready")


if __name__ == "__main__":
    if sys.argv[0].split('/')[-1] == 'hooks.py':
        del sys.argv[0]
    hooks.execute(sys.argv)
